@@ -322,8 +322,12 @@ class Mtrk:
 			dd - sig denominator, 2=quarter note, 3=eighth, 
 			cc - midi clocks/tick
 			bb - # of 32nd notes in quarter (normally 8)
+			
+			This is only called by timeSig.set(). Don't
+			call this directly since the timeSig.set() checks for
+			duplicate settings.
 		"""
-	
+		
 		cmd = chr(0xff) + chr(0x58)
 		self.delDup(offset, cmd)
 		self.addToTrack(offset, cmd + chr(0x04) + \
@@ -646,4 +650,21 @@ class Mtrk:
 		
 				
 
+class TimeSig:
+
+	def __init__(self):
+		self.lastsig = (None,None)
+		
+	def set(self, nn, dd):
+		if self.lastsig != (nn, dd):
+			gbl.mtrks[0].addTimeSig(gbl.tickOffset, nn, dd, 48, 8)
+			self.lastsig = (nn, dd)
+
+	def get(self):
+		return self.lastsig[:]
+
+
+timeSig = TimeSig()
+
+
 
