@@ -35,13 +35,15 @@ import MMAglobals;  gbl = MMAglobals
 from MMAchords import ChordNotes
 import MMAfile
 import MMAdocs
+import MMAmidi
+import MMAmidiIn
 from MMAauto  import loadGrooveDir
 from MMAalloc import trackAlloc
 from MMAcommon  import *
 import MMAtranslate
-import MMAlyric
+from MMAlyric import lyric
 import MMApatSolo
-import MMAmacro; macros = MMAmacro.Macros()
+from MMAmacro import macros
 import MMAmdefine
 
 def setDebugMacro(n):
@@ -232,21 +234,32 @@ def repeat(ln):
 	while 1:
 		if act in ('REPEATEND', 'ENDREPEAT'):
 			if l:
-				error("REPEATEND does not take an argument.")
-			stack.extend(main)
-			stacknum.extend(mainnum)
-		
+				if len(l)>1:
+					error("%s takes only one (optional) arg." % act)
+				count=stoi(l[0], "%s takes an integer arg." % act)
+				if count==2:
+					warning("REPEATEND using default count value (2). "
+						"Did you mean 3 or more?")
+				if count<2:
+					error("%s count must be 2 or greater." % act)
+				if count>25:
+					warning("%s is a large value for %s" % (count, act) )
+			else:
+				count=2	
+
 			if not ending:
+				count += 1
+			for c in range(count-1):
 				stack.extend(main)
 				stacknum.extend(mainnum)
-
+		
 			gbl.inpath.push(stack, stacknum)
 			break
 							
 		elif act == 'REPEATENDING':
 			ending = 1
 			if len(l) > 1:
-				error("REPEATENDING only takes one arg.")
+				error("REPEATENDING only takes one (optional) arg.")
 			if len(l) == 1:
 				count=stoi(l[0], 
 					"REPEATENDING takes an integer arg.")
@@ -667,7 +680,8 @@ def grooveDefineDo(slot):
 
 	gbl.settingsGroove[slot] = {'SEQSIZE': gbl.seqSize,
 			'QPERBAR': gbl.QperBar,
-			'SEQRND':  seqRnd  }
+			'SEQRND':  seqRnd,
+			'TIMESIG': MMAmidi.timeSig.get()  }
 			
 
 def groove(ln):
@@ -725,6 +739,10 @@ def grooveDo(slot):
 	gbl.seqSize   = g['SEQSIZE']
 	gbl.QperBar   = g['QPERBAR']
 	seqRnd        = g['SEQRND'] 
+	n,d           = g['TIMESIG']
+	if n or d:
+		MMAmidi.timeSig.set(n,d)
+		
 	
 	for n in gbl.tnames.values():
 		n.restoreGroove(slot)
@@ -1061,7 +1079,8 @@ def setTimeSig(ln):
 	else:
 		error("Unknown value for timesig denominator")
 	
-	gbl.mtrks[0].addTimeSig(gbl.tickOffset, nn, dd, 48, 8)
+	MMAmidi.timeSig.set(nn,dd)
+
 
 	
 	
@@ -1362,6 +1381,20 @@ def trackRiff(name, ln):
 		
 	gbl.tnames[name].setRiff(ln)
 
+def trackMultiRiff(name, ln):
+	""" Set a multi-riff for a track.
+	
+		Same as trackRiff but this function stacks riffs.
+	"""
+	
+	ln=' '.join(ln)
+	
+	v = gbl.tnames[name].vtype
+	if v != 'SOLO' and v != 'MELODY':
+		ln = ln.rstrip('; ')
+		
+	gbl.tnames[name].setMultiRiff(ln)
+
 
 def deleteTrks(ln):
 	""" Delete a track and free the MIDI track. """
@@ -1379,6 +1412,9 @@ def deleteTrks(ln):
 		if tr.channel:
 			tr.doMidiClear()
 			tr.clearPending()
+			
+			if tr.riff:
+				warning("%s has pending RIFF(s)" % name)			
 			gbl.midiAvail[tr.channel] -= 1
 
 			# NOTE: Don't try deleting 'tr' since it's just a copy!!
@@ -1487,14 +1523,6 @@ def trackMallet(name, ln):
 		
 	gbl.tnames[name].setMallet(ln)
 
-def trackMalletDK(name, ln):
-	""" Set repeating-mallet decay rate for solo/melody track. """
-	
-	if len(ln) != 1:
-		error("Use: %s MalletDecay <RATE>." % name)
-		
-	gbl.tnames[name].setMalletDK(ln[0])
-
 	
 def trackRtime(name, ln):
 	""" Set random timing for specific track. """
@@ -1866,9 +1894,15 @@ def mdata(ln):
 		rptcount = 1
 	
 	ln = ' '.join(ln)
-	ln, lyrics = MMAlyric.lyric.extract(ln, barNum, rptcount)	
-	ln, solo = MMApatSolo.extractSolo(ln, rptcount)
+
+	# Set lyrics from [stuff] in the current line or 
+	# stuff previously stored with LYRICS SET.
 	
+	ln, lyrics = lyric.extract(ln, barNum, rptcount)
+	
+	# Same for solo: {stuff} or RIFF(S). The solo variable is not used.
+	
+	ln, solo = MMApatSolo.extractSolo(ln, rptcount)
 	
 	# Pad out chords to correct number of '/'s
 	
@@ -1973,6 +2007,15 @@ def mdata(ln):
 
 		if seqRnd:
 			gbl.seqCount = random.randrange(gbl.seqSize)
+			
+		""" Special loop for solo/harmony tracks... We take the
+			first track in autosolotracks[] and if it's assigned
+			and has data we duplicate the data to any other solo/melody
+			tracks in the list which are set as harmonyonly.
+		"""
+		
+		MMApatSolo.copySoloToHarmony()
+
 
 		""" Process each track. It is important that the track classes
 			are written so that the ctable passed to them IS NOT MODIFIED.
@@ -2047,10 +2090,11 @@ simpleFuncs={
 	'INC':                 macros.varinc,
 	'INCLUDE':             include,
 	'KEYSIG':              keySig,
-	'LYRIC':               MMAlyric.lyric.option,
+	'LYRIC':               lyric.option,
 	'MIDIDEF':             mdefine,
 	'MIDI':                rawMidi,
 	'MIDIFILE':            setMidiFileType,
+	'MIDIINC':             MMAmidiIn.midiinc,
 	'MMAEND':              mmaend,
 	'MMASTART':            mmastart,
 	'MSET':                macros.msetvar,
@@ -2117,6 +2161,7 @@ trackFuncs={
 	'PORTAMENTO':          trackGlis,
 	'RANGE':               trackRange,
 	'RIFF':                trackRiff,
+	'RIFFS':               trackMultiRiff,
 	'RSKIP':               trackRskip,
 	'RTIME':               trackRtime,
 	'RVOLUME':             trackRvolume,
