@@ -77,7 +77,7 @@ class PC:
 		self.saveVols = {}
 		self.ssvoice = -1	 # Track the voice set for the track
 		self.smidiVoice = () # Track MIDIVoice cmds to avoid dups
-		self.midiSent = 0
+		self.midiSent = 0    # if set, MIDICLEAR invoked.
 				
 		self.artic = (90,)	# Note length clip adjustment
 
@@ -87,12 +87,11 @@ class PC:
 
 		self.midiPending = []		
 
-		self.riff = ''
+		self.riff = []
 				
 		self.disable = 0
 		
 		if self.vtype == 'DRUM':
-			self.ssvoice = 0
 			self.setChannel('10')
 			if not gbl.mtrks[self.channel].trackname:
 				gbl.mtrks[self.channel].addTrkName(0, 'Drum')
@@ -133,6 +132,9 @@ class PC:
 		
 		self.compress = seqBump(tmp)
 		
+		if self.vtype not in ("CHORD", "ARPEGGIO"):
+			warning ("Compress is ignored in %s tracks." % self.vtype)
+			
 		if gbl.debug:
 			print "Set %s Compress to " % self.name,
 			printList(ln)
@@ -321,6 +323,10 @@ class PC:
 						
 		self.chordLimit = n
 		
+		if self.vtype not in ("CHORD", "ARPEGGIO"):
+			warning ("Limit is ignored in %s tracks." % self.vtype)
+			
+
 		if gbl.debug:
 			print "Set %s ChordLimit to %s" % (self.name, n)
 
@@ -422,7 +428,7 @@ class PC:
 			print "MIDI Channel %s assigned to %s." % (self.channel, self.name)
 			
 
-	def setChShare(self, ln):
+	def setChShare(self, ln, warn=1):
 		""" Share midi-channel setting. """
 
 		if self.channel:	# If channel already assigned, ignore
@@ -458,9 +464,9 @@ class PC:
 		# Actually do the assignment
 		
 		self.channel = schannel
-		
+
 		warning("%s and %s now share MIDI channel %s" %
-			(sc, self.name, self.channel))
+				(sc, self.name, self.channel))
 
 		# Update the avail. lists
 			
@@ -473,6 +479,9 @@ class PC:
 		
 		self.midiPending.append(( "CVOLUME", gbl.tickOffset, v) )
 
+		if gbl.debug:
+			print "Set %s MIDIChannelVolume to %s" % (self.name, v)
+
 				
 	def setPan(self, ln):
 		""" Set MIDI Pan for this track. """
@@ -484,6 +493,10 @@ class PC:
 
 		self.midiPending.append( ("PAN", gbl.tickOffset, v))
 
+		if gbl.debug:
+			print "Set %s MIDIPan to %s" % (self.name, v)
+
+
 		
 	def setGlis(self, ln):
 		""" Set MIDI Glis for this track. """
@@ -495,6 +508,10 @@ class PC:
 			
 		self.midiPending.append( ("GLIS", gbl.tickOffset, v))
 
+		if gbl.debug:
+			print "Set %s MIDIPortamento to %s" % (self.name, v)
+
+
 
 	def setStrum(self, ln):
 		""" Set Strum time. """
@@ -534,6 +551,7 @@ class PC:
 		
 		self.disable = 0
 		self.ssvoice = -1
+
 		if gbl.debug:
 			print "%s Enabled" % self.name
 
@@ -624,6 +642,7 @@ class PC:
 		""" Enable random pattern selection from sequence."""
 		
 		self.seqRnd = 1
+
 		if gbl.debug:
 			print "Set %s SeqRnd." % self.name
 
@@ -632,6 +651,7 @@ class PC:
 		""" Disable random pattern selection from sequence."""
 
 		self.seqRnd = 0
+
 		if gbl.debug:
 			print "Cleared %s SeqRnd." % self.name
 
@@ -651,6 +671,10 @@ class PC:
 			tmp.append(n)
 			
 		self.direction = seqBump( tmp )
+		if self.vtype == 'SCALE':
+			self.lastChord = None
+			self.lastNote = -1
+	
 
 		if gbl.debug:
 			print "Set %s Direction to " % self.name,
@@ -662,7 +686,7 @@ class PC:
 		
 		ln = self.lnExpand(ln, "ScaleType")
 		
-		vald = ('MAJOR', 'MINOR', 'CHROMATIC', 'AUTO')
+		vald = ( 'CHROMATIC', 'AUTO')
 		tmp = []
 		
 		for n in ln:
@@ -675,7 +699,8 @@ class PC:
 		if self.vtype == 'SCALE':
 			self.scaleType = seqBump( tmp )
 	
-			self.notes = None		
+			self.lastChord = None
+			self.lastNote = -1
 	
 		if gbl.debug:
 			if self.vtype=='SCALE':
@@ -710,6 +735,9 @@ class PC:
 			
 		self.invert = seqBump( tmp )
 
+		if self.vtype not in ("CHORD", "ARPEGGIO"):
+			warning ("Invert is ignored in %s tracks." % self.vtype)
+		
 		if vwarn:
 			warning("Setting both Voicing Mode and Invert is not a good idea")
 		
@@ -762,9 +790,7 @@ class PC:
 			
 		if gbl.debug:
 			print "Set %s Harmony to" % self.name,
-			for i in self.harmony:
-				print i,
-			print
+			printList(self.harmony)
 
 
 	def setHarmonyOnly(self, ln):
@@ -789,9 +815,7 @@ class PC:
 			
 		if gbl.debug:
 			print "Set %s HarmonyOnly to" % self.name,
-			for i in self.harmonyOnly:
-				print i,
-			print
+			printList(self.harmonyOnly)
 
 
 	def setSeqSize(self):
@@ -992,11 +1016,6 @@ class PC:
 	
 		error("Mallet not supported in %s track." % self.name)
 		
-	def setMalletDecay(self, ln):
-		""" Set the note repeat/mallet decay value. """
-	
-		error("MalletDecay not supported in %s track." % self.name)
-
 		
 	def setAccent(self, ln):
 		""" Set the accent values. """
@@ -1336,7 +1355,7 @@ class PC:
 					
 		if self.riff:
 			warning("%s sequence clear deleting unused riff" % self.name)
-		self.riff = None
+		self.riff = []
 		
 		if self.vtype == 'CHORD':
 			self.voicing   = Voicing()
@@ -1385,17 +1404,33 @@ class PC:
 
 		if self.riff:
 			warning("%s overwriting unused riff." % self.name)
+			self.riff=[]
 			
 		if len(ln) == 1 and (ln[0] in ('Z','z','-')):
-			self.riff = None
+			self.riff = []
 			if gbl.pshow:
 				print "%s Riff cancelled." % self.name
 				
 		else: 
-			self.riff = self.defPatRiff(ln)
+			self.riff = [self.defPatRiff(ln)]
 			if gbl.pshow:
 				print "%s Riff:" % self.name
-				self.printPattern(self.riff)
+				self.printPattern(self.riff[-1])
+
+
+	def setMultiRiff(self, ln):
+		""" Define and set a multi-riff. 
+		
+			Same as a riff, but this stacks.
+		"""
+
+		if len(ln) == 1 and (ln[0] in ('Z','z','-')):
+			error("Can't delete a Riff with 'z' or '-' in multi-mode")
+
+		self.riff.append(self.defPatRiff(ln))
+		if gbl.pshow:
+			print "%s RiffAppend:" % self.name
+			self.printPattern(self.riff[-1])
 
 		
 
@@ -1584,8 +1619,7 @@ class PC:
 		""" Get pattern for this sequence. Either a Riff or a Pattern. """
 		
 		if self.riff:
-			pattern = self.riff
-			self.riff = None
+			pattern = self.riff.pop(0)
 	
 		else:
 			if not self.sequence:
@@ -1608,17 +1642,22 @@ class PC:
 		
 		self.clearPending()	
 
-		# Do the MIDIVoice pattern if it's different...
-		
+		""" 1st pass for MIDIVOICE. There's a separate slot for
+			each bar in the sequence, plus the data can be sent
+			before or after 'voice' commands. This first loop
+			sends MIDIVOICE data with an offset of 0. Note, we
+			don't set the value for 'self.smidiVoice' until we
+			do this again, later. All this is needed since some
+			MIDIVOICE commands NEED to be sent BEFORE voice selection,
+			and others AFTER.
+		"""
 		
 		if self.midiVoice:
 			v = self.midiVoice[sc]
 			if v and v != self.smidiVoice:
 				for i in v:
-					gbl.mtrks[self.channel].addCtl(gbl.tickOffset, i[1])
-				self.smidiVoice = v
-				self.midiSent = 1
-
+					if not i[0]:
+						gbl.mtrks[self.channel].addCtl(gbl.tickOffset, i[1])
 		
 		# Set the voice in the midi track if not previously done.
 
@@ -1626,10 +1665,30 @@ class PC:
 		if v != self.ssvoice:
 			gbl.mtrks[self.channel].addProgChange( gbl.tickOffset,  v)
 			self.ssvoice = v
-			
+		
+			# Mark ssvoice also in shared tracks
+
+			for a in gbl.midiAssigns[self.channel]:
+				if gbl.tnames.has_key(a):
+					gbl.tnames[a].ssvoice = v
+						
 			if gbl.debug:
 				print "Track %s Voice %s inserted." \
 					% (self.name, MMAmidi.valueToInst(v) )
+
+		""" Our 2nd stab at MIDIVOICE. This time any sequences
+			with offsets >0 are sent. AND the smidiVoice and midiSent
+			variables are set.
+		"""
+		
+		if self.midiVoice:
+			v = self.midiVoice[sc]
+			if v and v != self.smidiVoice:
+				for i in v:
+					if i[0]:
+						gbl.mtrks[self.channel].addCtl(gbl.tickOffset, i[1])
+				self.smidiVoice = v
+				self.midiSent = 1  # used by MIDICLEAR	
 					
 
 		# Do MIDISeq for this voice
